#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan  4 18:05:35 2024
@author: David Alberto (www.astrolabe-science.fr)

Ce script produit le tracé d'un cadran solaire cylindrique dit "de berger".
Le document imprimé doit être collé autour d'un cylindre ;
une tige horizontale doit surplomber le tracé.
Personnaliser la latitude d'utilisation, le diamètre du cylindre,
et la longueur du style en surplomb.
Le style doit correspondre au bord supérieur du tracé.
Pour régler la date, soit le style est mobile et se place au-dessus du trait
de date, soit c'est le tracé qui est mobile.
Une fois la date réglée, il faut orienter le style vers le Soleil ; son
ombre doit être verticale.
"""

import matplotlib.pyplot as plt
import numpy as np
from math import sin, cos, tan, radians, degrees, pi, asin, acos
import datetime as dt
import locale
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8') # dates en français

# -------------- Paramètres à personnaliser ---------------------
lat = 55 # latitude en degrés
# diamètre en cm :
D = 5.75 # canette soda forme haute
# D = 7.53 # boîte chips Pringles
a = 9  # longueur du style en surplomb (cm)

prop_heures = dict(lw=0.6, color='mediumblue')
prop_demies = dict(lw=0.4, color='gray')
prop_chiffres = dict(bbox=dict(boxstyle="round", fc='white', alpha=0.7))

plt.rcParams["font.family"] = "Roboto Condensed"

#  dimensions de la figure et du graphique :
inch = 2.54 # conversion des cm en pouces
fig_width = 29.7 / inch 
fig_height = 21 / inch
# format A4, mais à revoir si le cylindre est trop grand
marge_G = 1 / inch # cm de marge gauche

#  --------------------------------------------------------------------

eps = radians(23.439)# obliquité de l'écliptique
lat_r = radians(lat)
liste_J = np.arange(1,366,0.1)
Perim = pi*D # périmètre du cylindre

largeur_axe = Perim / inch
marge_D = fig_width - marge_G - largeur_axe
marge_B = marge_G

#  --------------------------------------------------------------------
# FONCTIONS

def declinaison(J, deg = False):
    """
    Parameters
    ----------
    J : TYPE : entier
        DESCRIPTION : rang du jour dans l'année.
    deg : booléen (True si on veut la valeur en degrés)
    Returns
    -------
    Déclinaison du Soleil (radians par défaut, degrés si deg=True).
    Cette modélisation ne donne qu'une indication, de précision suffisante
    pour ce cadran solaire.
    """
    long = (2*pi*(J-81))/365.2422
    decl = asin(sin(eps)*sin(long))
    if deg:decl = degrees(decl)
    return decl

def hauteur(J, H, deg = False):
    """
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    deg : booléen (True si on veut la valeur en degrés)
    ------------------
    Renvoie la hauteur du Soleil en radians, par défaut, degrés si deg = True.
    """
    decl = declinaison(J)
    sinh = sin(lat_r)*sin(decl) + cos(lat_r)*cos(decl)*cos(H)
    h = asin(sinh)
    if deg:
        h = degrees(h)
    return h

def verticale(J, H):
    """
    J (entier) : rang du jour dans l'année (1 à 365)
    H (float) : angle horaire du Soleil (radians)
    Renvoie la position verticale de l'ombre de l'extrếmité du style,
    depuis le bord supérieur du tracé.
    """
    y = - a * tan(hauteur(J, H))
    return y

verticale = np.vectorize(verticale) # vectorisation de la fonction

def rang(date):
    """
    Parameters
    ----------
    jour, mois : TYPE : deux entiers
        DESCRIPTION : n° du jour dans le mois, n° du mois dans l'année
    Returns
    -------
    Renvoie le rang du jour dans l'année (entier).
    Année non bissextile.
    """
    unjanvier = dt.datetime(2026,1,1)
    J = (date - unjanvier).days + 1
    return J

rang = np.vectorize(rang) # vectorisation de la fonction
#  FIN DES FONCTIONS
#  ---------------------------------------------------------------

ymin = verticale(172,0) # ordonnée minimale
hauteur_axe = (- ymin + 0.5) / inch
marge_H = fig_height - marge_B - hauteur_axe

# définition de la figure et du graphique :
fig = plt.figure(figsize=(fig_width, fig_height))



ax = plt.subplot(111)
ax.set(
       ylim=(ymin-0.5,0),
       xlim=(0,366),
       xticks=([]),
       yticks=([]),
       # aspect='equal'
       )

Hmax = degrees(acos(-tan(lat_r)*tan(eps))) # angle horaire maximal au 21 juin

# tracé des courbes horaires :
for H in np.arange(0, Hmax, 15):
    heure_soir = int(12 + H/15)
    heure_matin = int(12 - H/15)
    H_rad = radians(H)
    hauteurs = verticale(liste_J, H_rad)
    ax.plot(liste_J, hauteurs, **prop_heures)
    y0 = verticale(172, H_rad)
    if H != 0 and y0 < -0.5:
        ax.text(172, verticale(172, H_rad), f"{heure_matin}-{heure_soir}",
            va='center',ha='center', **prop_chiffres)    
    ax.text(172, verticale(172, 0), "12",
            va='center',ha='center', **prop_chiffres)

for H in np.arange(7.5, Hmax, 15):# demi-heures
    H_rad = radians(H)
    hauteurs = verticale(liste_J, H_rad)
    ax.plot(liste_J, hauteurs, **prop_demies)

#  ----------------------------------------------------

# graduations du calendrier :
liste_mois = [dt.datetime(2026, m, 1) for m in range(1,13)]

for jour in liste_mois:
    J = rang(jour)
    ax.axvline(J, c='gray', lw=0.4)
    date = jour.strftime("%B")
    ax.text(J, ymin, date, ha='left', rotation=90)
    ax.axvline(J+9, c='gray', lw=0.2, alpha=0.5)
    ax.axvline(J+19, c='gray', lw=0.2, alpha=0.5)
    if jour.month !=2:
        ax.axvline(J+29, c='gray', lw=0.2, alpha=0.5)

#  -------------------------------------------------------------

ax.text(0.999, 0.01, f"latitude {lat}° - diamètre {D} cm - style {a} cm", transform = ax.transAxes, c='firebrick',
        fontsize=14, rotation=90, ha='right')

#  -------------------------------------------------------------
# ajustement des dimensions de l'axe :
# les valeurs "left", "right", "bottom" et "top" sont des % entre 0 et 1

plt.subplots_adjust(
    left = marge_G / fig_width,
    right = 1 - marge_D/fig_width,
    bottom= marge_B / fig_height,
    top = 1 - marge_H / fig_height,
)





fig.savefig("cadran_berger" + str(lat)+ ".pdf")










